# $Id: gamelist.pl,v 1.30 2000/04/11 16:38:14 hans Exp $

require 'sgml2.pl';
use game;
use series;
use company;
use machine;
use platform;
use emulator;

$infile = $ARGV[0];
$htmldir = $ARGV[1];

# Build hash to map ISO country codes to flag gif names.
open(COUNTRY, "<country.txt") || die("Couldn't open country.txt!");
while (<COUNTRY>) {
    ( $tag, $gif, @name ) = split;
    $name = join(" ", @name);
    $countries{$tag} = $name;
    $gifs{$tag} = $gif;
}
close(COUNTRY);


sub txt {
    local ($txt) = @_;

    $txt =~ s/\<.*?\>//gs;
    $txt =~ s/\n[ \t]+/ /gs;
    $txt =~ s/([a-zA-Z0-9-.,;:'*+?()\/&"! \n]{0,70})[ \n]/$1\n/g;
    print TXTFILE $txt;
}


sub data {
    local ($_) = @_;

    s/\{/\</gs;
    s/\}/\>/gs;
    s/\n+$//;
    s/\n\n/\n<p>\n/sg;
    s/\s*$//;
    s/^\s*//;
    $lastdata = $_;
}



sub link_to_gt {
    my ($type, $series) = @_;
    my (@res);

    foreach $t (split("/", $type)) {
	if ($gametypes{$t}) {
	    push(@res,
		 "<a href=\"" .
		 (($parsing_series || ($series eq "serfile")) ? "../" : "") . 
		 "gametypes.html#gametype_$t\">$gametypes{$t}</a>");
	} else {
	    push(@res, $type);
	}
    }
    return join("/", @res);
}

sub strip_name {
    my ($name) = @_;
    my ($stripped_name);

    ( $stripped_name = $name ) =~ s/([^\n]+)\n.*/$1/s;
    $stripped_name =~ s/\[.*\]//g;
    $stripped_name =~ s/\s+$//;
    $stripped_name =~ s/'//g; #' Stupid Perl mode
    $stripped_name =~ s/!//g;
    $stripped_name =~ s/\&\s*//g;
    $stripped_name =~ tr| /|__|;
    return $stripped_name;
}

sub commalist {
    my ( $lastbit, @list) = @_;

    if ($#list == 0) {
	return $list[0];
    } elsif ($#list == 1) {
	return join($lastbit, @list);
    } else {
	return join(", ", @list[0..$#list-2], 
		    join($lastbit, @list[$#list-1..$#list]));
    }
}

sub make_game_link {
    my ($machineline, $series) = @_;
    my ($machine, $comments);

    ($machine, $comments) = split(/\[/, $machineline);
    $machine =~ s/\s*$//;
    return "<a href=\"" .
	($series eq "serfile" ? "../machines/" : "machines/") .
	strip_name($machine) .
	".html\">$machine</a>" .
	($comments ? " [$comments" : "") .
	"\n";
}

sub print_page_header {
    my ($fh) = @_;

    open(HTMLHEADER, "<header.html") || die("Couldn't open header.html!");
    while ($line = <HTMLHEADER>) {
      print $fh $line;
    }
    close(HTMLHEADER);
}

sub print_subpage_header {
    my ($fh) = @_;

    open(HTMLSUBHEADER, "<subheader.html") || die("Couldn't open subheader.html!");
    while ($line = <HTMLSUBHEADER>) {
      print $fh $line;
    }
    close(HTMLSUBHEADER);
}

sub print_page_footer {
    my ($fh) = @_;

    open(HTMLFOOTER, "<footer.html") || die("Couldn't open footer.html!");
    while ($line = <HTMLFOOTER>) {
      print $fh $line;
    }
    close(HTMLFOOTER);
}

sub print_subpage_footer {
    my ($fh) = @_;

    open(HTMLSUBFOOTER, "<subfooter.html") || die("Couldn't open subfooter.html!");
    while ($line = <HTMLSUBFOOTER>) {
      print $fh $line;
    }
    close(HTMLSUBFOOTER);
}

sub print_game {
    my ($game, $series, $compname, $file) = @_;

    # Print a separator line before every game except the last one.
    if ($series eq "series") {
	print $file "<p><li>";
    } else {
	if ($noofgames--) {
	    print $file "<p><hr>\n\n";
	}
    }
    print $file "<a name=\"" . &strip_name($game->gamename) . "\">\n";
    print $file "<h3>";
    if ($game->cover) {
	print $file "<img align=right src=\"" .
	    ($series eq "serfile" ? "../images/" : "images/") .
	    $game->{"cover"} .
	    "\">";
    }
    print $file $game->gamename . "</h3>\n";
    print $file "Type: " . &link_to_gt($game->type, $series) . "\n" 
	if ($game->type);
    print $file "Written " if ($game->year || $game->by || $game->in);
    print $file $game->year if ($game->year);
    print $file " by " . commalist(" and ", $game->by) if ($game->by);
    print $file " in " . $game->in if ($game->in);
    print $file ".\n" if ($game->year || $game->by || $game->in);
    if ($game->for) {
	print $file "<br>Runs on:\n<ul>\n";
	foreach $g ($game->for) {
	    print $file "<li>" . make_game_link($g, $series);
	}
	print $file "</ul>\n";
    }
    print $file "<br>Notes: " . $game->note . "\n" if ($game->note);
    $page = $game->page;
    if ($page) {
	( $page, $pagename ) = split("", $page); 
	print $file 
	    "<br>More information about <a href=\"$page\">$pagename</a>.";
    }
    print $file "<p>Comments: " . $game->comm . "\n" if ($game->comm);
    print $file "<p>Package blurb:<blockquote> " . $game->blurb . "</blockquote>\n"
	if ($game->blurb);
    print $file "<p>Here is a <a href=\"" .
	$game->review . "\">review of " . $game->gamename . "</a>\n" 
	    if ($game->review);
    print $file "<p>" . $game->gamename . " is available for <a href=\"" .
	$game->download . "\">off-site download</a>\n"
	    if ($game->download);
    print $file "For more information, see " . 
	commalist(" and ", $game->see) . "\n" if ($game->see);
    print $file "<br clear=all>\n";
}

sub txt_print_game {
    my ($game, $series, $compname) = @_;
    my ($toprint);

    # Print a separator line before every game except the last one.
    if ($series eq "series") {
	&txt("\n * ");
    } else {
	if ($noofgames--) {
	    &txt("\n" . '-' x 70 . "\n");
	}
    }
    &txt($game->gamename . "\n");
    &txt("Type: " . ($game->type) . "\n") if ($game->type);
    $toprint = "Written" if ($game->year || $game->by || $game->in);
    $toprint .= " " . $game->year if ($game->year);
    $toprint .= " by " . commalist(" and ", $game->by) if ($game->by);
    $toprint .= " in " . $game->in if ($game->in);
    $toprint .= ".\n" if ($game->year || $game->by || $game->in);
    &txt($toprint);
    if ($game->for) {
	&txt("\nRuns on:\n");
	foreach $g ($game->for) {
	    &txt(" * $g\n");
	}
	&txt("\n");
    }
    &txt("Notes: " . $game->note . "\n") if ($game->note);
#    $page = $game->page;
#    if ($page) {
#	( $page, $pagename ) = split("", $page); 
#	print $file 
#	    "<br>More information about <a href=\"$page\">$pagename</a>.";
#    }
    &txt("Comments: " . $game->comm . "\n") if ($game->comm);
    &txt("Package blurb: " . $game->blurb . "\n") if ($game->blurb);
    &txt("For more information, see " . 
	commalist(" and ", $game->see) . ".\n") if ($game->see);
    &txt("\n");
}

sub print_series {
    my ($series) = @_;
    my ($fh) = @_;

    print $file "<hr><h2>";
    if ($series->cover) {
	print $file "<img align=right src=\"images/" . $series->{"cover"} .
	    "\">";
    }
    if ($series->serfile) {
	print $file "<a href=\"" . $series->serfile . "\">" .
	    $series->sername . 
	    "</a></h2>\n";
	open(SERFILE, ">$htmldir/" . $series->serfile) ||
	    die("Couldn't open $htmldir/" . $series->serfile . "!");

	print SERFILE "<html>\n<head>\n<title>" . $series->sername . 
	    "</title>\n";
        open(HTMLSUBHEADER, "<subheader.html") || die("Couldn't open subheader.html!");
        while ($line = <HTMLSUBHEADER>) {
          print SERFILE $line;
        }
        close(HTMLSUBHEADER);
	print SERFILE "<h2>";
	if ($series->cover) {
	    print SERFILE "<img align=right src=\"images/" . 
		$series->{"cover"} . "\">";
	}
	print SERFILE $series->sername . "</h2>\n";
	print SERFILE "<p>Go back to <a href=\"../" . 
	    &strip_name($company->compname) .
	    ".html\">" . $company->compname . "</a>.\n";
	print SERFILE "<p>" . $series->serdesc . "\n" if ($series->serdesc);
	print SERFILE "<p>For more information, see " . 
	    commalist(" and ", $series->see) . ".\n" if ($series->see);
	print SERFILE "<br clear=all>\n";
    } else {
	print $file $series->sername . "</h2>\n";
    }
    print $file "<p>" . $series->serdesc . "\n" if ($series->serdesc);
    print $file "<p>For more information, see " . 
	commalist(" and ", $series->see) . ".\n" if ($series->see);
    print $file "<br clear=all>\n";
    if ($series->serfile) {
	foreach $game ($series->game) {
	    &print_game($game, "serfile", $company->compname, SERFILE);
	}
	print_subpage_footer(SERFILE);
	close(SERFILE);
    } else {
	print $file "<ul>\n";
	foreach $game ($series->game) {
	    &print_game($game, "series", $company->compname, $file);
	}
	print $file "</ul>\n";
    }
}

sub txt_print_series {
    my ($series) = @_;

    &txt($series->sername . "\n");
    &txt("\n" . $series->serdesc . "\n") if ($series->serdesc);
    &txt("\nFor more information, see " . 
	commalist(" and ", $series->see) . ".\n") if ($series->see);
    foreach $game ($series->game) {
	&txt_print_game($game, "series");
    }
}



sub E_GAMELIST {
    my ( $date );

    $no_companies = 0;

    #chomp($date = `date '+%y%m%d'`);
    open(NAMES, ">$htmldir/names.html") || die("Couldn't open $htmldir/names.html!");
    $startletter = "A";
    print NAMES "<!-- This file generated by gamelist.pl $date. -->\n";
#    print NAMES "<insert file=defines>\n";
    print NAMES "<html><head><title>Adventureland: Company Names</title>\n";
    print_page_header(NAMES);
    print NAMES "<center><a href=\"\#A\">A</a>   <a href=\"\#B\">B</a>   <a href=\"\#C\">C</a>   <a href=\"\#D\">D</a>   \n";
    print NAMES "<a href=\"\#E\">E</a>   <a href=\"\#F\">F</a>   <a href=\"\#G\">G</a>   <a href=\"\#H\">H</a>  \n";
    print NAMES "<a href=\"\#I\">I</a>   <a href=\"\#J\">J</a>   <a href=\"\#K\">K</a>   <a href=\"\#L\">L</a>  \n";
    print NAMES "<a href=\"\#M\">M</a>   <a href=\"\#N\">N</a>   <a href=\"\#O\">O</a>   <a href=\"\#P\">P</a>  \n";
    print NAMES "<a href=\"\#Q\">Q</a>   <a href=\"\#R\">R</a>   <a href=\"\#S\">S</a>   <a href=\"\#T\">T</a>  \n";
    print NAMES "<a href=\"\#U\">U</a>   <a href=\"\#V\">V</a>   <a href=\"\#W\">W</a>   <a href=\"\#X\">X</a>  \n";
    print NAMES "<a href=\"\#Y\">Y</a>   <a href=\"\#Z\">Z</a></center><hr>\n";
    print NAMES "<a name=\"" . $startletter . "\">\n";
    open(SEARCH, ">$htmldir/search.dat") || die("Couldn't open $htmldir/search.dat!");
    open(TXTFILE, ">$htmldir/gamelist.txt") || die("Couldn't open $htmldir/gamelist.txt");
    &txt("Adventureland - The guide to interactive fiction\n\n");
    &txt("This file contains information about adventure games and the companies that made them. If you have more information on any of the games in the list or if you find errors or omissions in the file, please let me know.\n\n");
    &txt("Adventureland was created by Hans Persson\n");
    &txt("Stefan Meier\n");
    &txt("adventureland\@if-legends.org\n\n");
    &txt("This version created " . localtime() . "\n\n\n");
    &parse_content;
    print_page_footer(NAMES);
    close(NAMES);
    close(SEARCH);
    close(TXTFILE);
}

sub E_GTYPES {
    open(GAMETYPES, ">$htmldir/gametypes.html") ||
	die("Couldn't open $htmldir/gametypes.html!");
    print GAMETYPES "<html><head><title>Adventure Game Types</title>\n";
    &print_page_header(GAMETYPES);
    &txt("Adventure Game Types\n\n");
    &parse_content;
    print GAMETYPES "</ul>\n";
    &print_page_footer(GAMETYPES);
    close(GAMETYPES);
}

sub E_GTINTRO {
    $lastdata = "";
    &parse_content;
    $gtintro = $lastdata;
    print GAMETYPES "<h1><font color=\"#FF0000\">Adventure Game Types</font></h1>\n";
    print GAMETYPES "$gtintro<p>\n";
    print GAMETYPES "<ul>\n";
    &txt("$gtintro\n");
}

sub E_GAMETYPE {
    $gtshort = "";
    $gtlong = "";
    $gtdesc = "";
    &parse_content;
    print GAMETYPES "<li><a name=gametype_$gtshort>" .
	"<b><a href=\"search.cgi?type=$gtshort\">$gtlong</a> ($gtshort)</b><br>\n".
	"$gtdesc\n";
    $gametypes{$gtshort} = $gtlong;
    &txt("$gtlong ($gtshort)\n$gtdesc\n");
}

sub E_GTSHORT {
    $lastdata = "";
    &parse_content;
    $gtshort = $lastdata;
}

sub E_GTLONG {
    $lastdata = "";
    &parse_content;
    $gtlong = $lastdata;
}

sub E_GTDESC {
    $lastdata = "";
    &parse_content;
    $gtdesc = $lastdata;
}

sub E_COMPANY {
    my (@see);

    $company = new company();
    @games = ();

    $comploc = $attrib{LOC};
    $comploc =~ s/CDATA\s*//;
    $comploc =~ s/IMPLIED\s*//;
    &parse_content;
    # Print the company information.
    $file = "FILE";
    $stripped_compname = &strip_name($company->compname);
    open($file, ">$htmldir/$stripped_compname.html") ||
        die("Couldn't open $htmldir/$stripped_compname!");
    print $file "<!-- Generated by gamelist.pl " . localtime() . "-->\n";
    print $file "<html>\n<head>\n<title>" . $company->compname . 
	"</title>\n";
    &print_page_header($file);
    print $file "<h1>";
    print $file "<img align=right src=\"images/logos/" . $company->logo . 
	"\">" 
	if ($company->logo);
    print $file "<img align=right " .
	"src=\"images/flags/$gifs{$comploc}.gif\"" .
	"alt=\"[$countries{$comploc}]\">" if ($comploc);
    print $file $company->compname . "</h1>\n";
    if ($startletter lt substr($stripped_compname, 0, 1)) {
	$startletter = substr($stripped_compname, 0, 1);
	print NAMES "<a name=\"" . $startletter . "\">\n";
    }
    print NAMES "<a href=\"$stripped_compname.html\">" .
	$company->compname . "</a>" .
	($comploc ? " ($comploc)" : "") . "<br>\n";
    print $file $company->compdesc . "\n" if ($company->compdesc);
    print $file "<p>" . $company->compname . " has a homepage " . 
	" at <a href=\"" . 
	$company->url . "\">" . $company->url . "</a>.\n" if ($company->url);
    print $file "<p>There are inofficial sites about " . $company->compname .
	":<ul>" if ($company->link);
    @linknames = $company->linkname;
    $i = 0;
    foreach $link ($company->link) {
	print $file "<li><a href=\"$link\">$linknames[$i]</a>";
	$i++;
    }
    print $file "</ul>" if ($company->link);
    print $file "<p>" . $company->compname . 
	" can be reached through " . commalist(" or ", $company->emaillink) .
	".\n" if ($company->email);
    print $file "<p>They are or have been distributed by " . 
	commalist(" and ", $company->distby) . ".\n" if ($company->distby);
    print $file "<p>They distribute " . commalist(" and ", $company->dist) .
	".\n" if ($company->dist);
    print $file "<p>For more information, see " . 
	commalist(" and ", $company->see) . ".\n" if ($company->see);
    print $file "<br clear=all>\n";
    # Print the games for the company.
    $noofgames = $#games + 1;
    foreach $item (@games) {
	if ($item->objtype eq "game") {
	    &print_game($item, "game", $company->compname, $file);
	} else {
	    &print_series($item);
	}
    }
    &print_page_footer($file);
    close($file);
    &txt("\n\n" . '=' x 70 . "\n\n") if ($no_companies++);
    &txt("\n\n" . $company->compname . "\n\n");
    &txt($company->compdesc . "\n\n") if ($company->compdesc);
    &txt("Homepage: " . $company->url . "\n") if ($company->url);
    &txt("Email: " . &commalist(" or ", $company->email) . ".\n")
	if ($company->email);
    &txt("They are distributed by " . 
	commalist(" and ", $company->distby) . ".\n") if ($company->distby);
    &txt("They distribute " . 
	commalist(" and ", $company->dist) . ".\n") if ($company->dist);
    &txt("For more information, see " . 
	commalist(" and ", $company->see) . ".\n") if ($company->see);
    &txt("\n");
    foreach $item (@games) {
	if ($item->objtype eq "game") {
	    &txt_print_game($item, "game", $company->compname);
	} else {
	    &txt_print_series($item);
	}
    }
}

sub E_EMAIL {
    $lastdata = "";
    $emailname = $attrib{NAME};
    $emailname =~ s/CDATA\s*//;
    $emailname =~ s/IMPLIED\s*//;
    &parse_content;
    $company->addemail($lastdata, $emailname);
}

sub E_LOGO {
    $lastdata = "";
    &parse_content;
    $company->addlogo($lastdata);
}

sub E_URL {
    $lastdata = "";
    &parse_content;
    $company->addurl($lastdata);
}

sub E_LINK {
    $lastdata = "";
    $linkname = $attrib{NAME};
    $linkname =~ s/CDATA\s*//;
    $linkname =~ s/IMPLIED\s*//;
    &parse_content;
    $linkname = $lastdata if ($linkname eq "");
    $company->addlink($lastdata, $linkname);
}

sub E_COMPDESC {
    $lastdata = "";
    &parse_content;
    $company->addcompdesc($lastdata);
}

sub E_COMPNAME {
    &parse_content;
    $company->addcompname($lastdata);
}

sub E_SERIES {
    $parsing_series = 1;
    $serfile = $attrib{SERFILE};
    $serfile =~ s/CDATA\s*//;
    $serfile =~ s/IMPLIED\s*//;
    $series = new series("serfile" => $serfile);
    &parse_content;
    push(@games, $series);
    $parsing_series = 0;
    $serfile = "";
}

sub E_SERNAME {
    &parse_content;
    $series->addsername($lastdata);
}

sub E_SERDESC {
    $lastdata = "";
    &parse_content;
    $series->addserdesc($lastdata);
    print $file "$serdesc<p>\n" if ($serdesc);
    print $file "<ol>\n";
}

sub E_GAME {
    my ($count, $see, $seeref);

    $parsing_game = 1;
    $game = new game("company" => $company->compname);
    &parse_content;
    if ($parsing_series) {
	$series->addgame($game);
	print SEARCH $company->compname . "\t" . 
	    ($series->serfile ? $series->serfile . "\t" :
	     &strip_name($company->compname) . ".html\t") .
	    $game->gamename . "\t" . 
	    &strip_name($game->gamename) . "\t" .
	    $game->year . "\t" .
	    $game->type . "\t" .
	    join(",", $game->by) . "\n";
	if ($series->serfile) {
	    $game->addserfile($series->serfile);
	}
    } else {
	push(@games, $game);
	print SEARCH $company->compname . "\t" . 
	    &strip_name($company->compname) . ".html\t" .
	    $game->gamename . "\t" . 
	    &strip_name($game->gamename) . "\t" .
	    $game->year . "\t" .
	    $game->type . "\t" .
	    join(",", $game->by) . "\n";
    }
    $parsing_game = 0;
}

sub E_GAMENAME {
    $lastdata = "";
    &parse_content;
    $game->addgamename($lastdata);
}

sub E_TYPE {
    $lastdata = "";
    &parse_content;
    $game->addtype($lastdata);
}

sub E_BY {
    my ($type);

    $lastdata = "";
    $type = $attrib{TYPE};
    $type =~ s/CDATA\s*//;
    $type =~ s/IMPLIED\s*//;
    &parse_content;
    push(@{$authors{$lastdata}}, $game);
    $lastdata .= " [$type]" if ($type);
    $game->addby($lastdata);
}

sub E_YEAR {
    $lastdata = "";
    &parse_content;
    $game->addyear($lastdata);
}

sub E_FOR {
    my ($mem, $note, $type, $by, $year, $comments);

    $lastdata = "";
    $mem = $attrib{MEM};
    $mem =~ s/CDATA\s*//;
    $mem =~ s/IMPLIED\s*//;
    $note = $attrib{NOTE};
    $note =~ s/CDATA\s*//;
    $note =~ s/IMPLIED\s*//;
    $type = $attrib{TYPE};
    $type =~ s/CDATA\s*//;
    $type =~ s/IMPLIED\s*//;
    $by = $attrib{BY};
    $by =~ s/CDATA\s*//;
    $by =~ s/IMPLIED\s*//;
    $year = $attrib{YEAR};
    $year =~ s/CDATA\s*//;
    $year =~ s/IMPLIED\s*//;
    &parse_content;
    $comments = 
	($type ? &link_to_gt($type) . "; " : "") .
	($year ? "$year; " : "") .
	($mem ? "$mem; " : "") .
	($by ? "$by; " : "") .
	($note ? "$note" : "");
    $comments =~ s/; $//;
    $comments = "[$comments]" if ($comments);
    $game->addfor($lastdata . $comments);
    if (! $machines{$lastdata}) {
	$machine = new machine("name" => $lastdata);
    } else {
	$machine = $machines{$lastdata};
    }
    $machine->addgame($game);
    $machines{$lastdata} = $machine;
}

sub E_COMM {
    $lastdata = "";
    &parse_content;
    $game->addcomm($lastdata);
}

sub E_IN {
    $lastdata = "";
    &parse_content;
    $game->addin($lastdata);
}

sub E_NOTE {
    $lastdata = "";
    &parse_content;
    $game->addnote($lastdata);
}

sub E_SEE {
    my ($seeref);

    $lastdata = "";
    $seeref = $attrib{REF};
    $seeref =~ s/CDATA\s*//;
    $seeref =~ s/IMPLIED\s*//;
    &parse_content;
    $lastdata .= "#$seeref" if ($seeref);
    if ($parsing_series && ! $parsing_game) {
	$series->addsee($lastdata);
    } elsif ($parsing_game) {
	$game->addsee($lastdata);
    } else {
	$company->addsee($lastdata);
    }
    push(@see, $lastdata);
}

sub E_BLURB {
    $lastdata = "";
    &parse_content;
    $game->addblurb($lastdata);
}

sub E_REVIEW {
    $lastdata = "";
    &parse_content;
    $game->addreview($lastdata);
}

sub E_DOWNLOAD {
    $lastdata = "";
    &parse_content;
    $game->adddownload($lastdata);
}

sub E_DIST {
    $lastdata = "";
    &parse_content;
    $company->adddist($lastdata);
}

sub E_DISTBY {
    $lastdata = "";
    &parse_content;
    $company->adddistby($lastdata);
}

sub E_COVER {
    $lastdata = "";
    &parse_content;
    if ($parsing_series && ! $parsing_game) {
	$series->addcover(&strip_name($company->compname) . "/" . $lastdata);
    } else {
	$game->addcover(&strip_name($company->compname) . "/" . $lastdata);
    }
}

sub E_PAGE {
    my ($pagename);
    $lastdata = "";
    $pagename = $attrib{PAGENAME};
    $pagename =~ s/CDATA\s*//;
    $pagename =~ s/IMPLIED\s*//;
    &parse_content;
    $game->addpage($lastdata, $pagename);
}

sub E_MACHINES {
    &parse_content;
}

sub E_MACHINE {
    &parse_content;
}

sub E_MACHNAME {
    $lastdata = "";
    &parse_content;
    $machname = $lastdata;
}

sub E_MACHY1 {
    my ($m);

    $lastdata = "";
    &parse_content;
    $m = new machine("name" => $machname);
    $m->addmachy1($lastdata);
    $machines{$machname} = $m;
}

sub E_MACHY2 {
    my ($m);

    $lastdata = "";
    &parse_content;
    if ($machines{$machname}) {
	$machines{$machname}->addmachy2($lastdata);
    } else {
	$m = new machine("name" => $machname);
	$m->addmachy2($lastdata);
	$machines{$machname} = $m;
    }
}

sub E_MACHDESC {
    my ($m);

    $lastdata = "";
    &parse_content;
    if ($machines{$machname}) {
	$machines{$machname}->addmachdesc($lastdata);
    } else {
	$m = new machine("name" => $machname);
	$m->addmachdesc($lastdata);
	$machines{$machname} = $m;
    }
}

# Functions for parsing platforms.

sub E_PLATFRMS {
    &parse_content;
}

sub E_PLATFORM {
    my ($filename);

    $filename = $attrib{FILENAME};
    $filename =~ s/CDATA\s*//;
    $filename =~ s/IMPLIED\s*//;
    &parse_content;
    $platform = new platform("platname" => $platname,
			     "platdesc" => $platdesc);
    $platform->addfilename($filename) if ($filename);
    $platforms{$platname} = $platform;
}

sub E_PLATNAME {
    $lastdata = "";
    &parse_content;
    $platname = $lastdata;
}

sub E_PLATDESC {
    $lastdata = "";
    &parse_content;
    $platdesc = $lastdata;
}

# Functions for parsing emulators.

sub E_EMULS {
    &parse_content;
}

sub E_EMULATOR {
    &parse_content;
    $emulators{$emulator->emulname} = $emulator;
}

sub E_EMULNAME {
    $lastdata = "";
    &parse_content;
    $emulator = new emulator("emulname" => $lastdata);
}

sub E_EMULYEAR {
    $lastdata = "";
    &parse_content;
    $emulator->addemulyear($lastdata);
}

sub E_EMULHOME {
    $lastdata = "";
    &parse_content;
    $emulator->addemulhome($lastdata);
}

sub E_EMULDESC {
    $lastdata = "";
    &parse_content;
    $emulator->addemuldesc($lastdata);
}

sub E_PLAYS {
    $lastdata = "";
    &parse_content;
    $emulator->addplays($lastdata);
}


&sgml_parse;

open(MACHINES, ">$htmldir/machines/index.html") ||
    die("Couldn't open $htmldir/machines/index.html!");
print MACHINES "<!-- This file generated by gamelist.pl -->\n";
#print MACHINES "<insert file=../defines>\n";
print MACHINES "<html><head><title>Adventureland: Computer Index</title>\n";
print_subpage_header(MACHINES);
print MACHINES "<h1><font color=\"#FF0000\">Computer Index</font></h1>\n";
print MACHINES "<ul>\n";
foreach $machinename (sort(keys(%machines))) {
    $lastcompany = "";
    $no_companies = 0;
    $machine = $machines{$machinename};
    @games = $machine->games;
    $no_games = $#games + 1;
    print MACHINES "<li><a href=\"" . &main::strip_name($machine->name) .
	".html\">" . $machine->name . "</a> (" . $no_games . ")\n";
    open(THISMACHINE, 
	 ">$htmldir/machines/" .
	 &main::strip_name($machine->name) . ".html") ||
	     die("Couldn't open $htmldir/machines/" . 
		 &main::strip_name($machine->name));
    print THISMACHINE "<!-- This file generated by gamelist.pl -->\n";
#    print THISMACHINE "<insert file=../defines>\n";
    print THISMACHINE "<html><head><title>Adventureland: " .
	$machine->name . "</title>\n";
    print_subpage_header(THISMACHINE);
    print THISMACHINE "<h1>" . $machine->name . " (" .
	($machine->machy1 ? $machine->machy1 : "?" ) . " - " .
	($machine->machy2 ? $machine->machy2 : "?" ) . ")</h1>\n";
    print THISMACHINE "<p>" . $machine->machdesc . "\n\n"
	if ($machine->machdesc);
    print THISMACHINE "<p>There " .
	($no_games == 1 ? "is 1 game " : "are $no_games games ") .
	"listed for " . $machine->name . " in this list.\n";
    print THISMACHINE "<ul>\n";
    foreach $game (@games) {
	$company = $game->company;
	if ($company ne $lastcompany) {
	    if ($no_companies++) {
		print THISMACHINE "</ul>\n";
	    }
	    print THISMACHINE "<li>" . 
		&main::company::compnametolink($company, "../") . 
		    "\n<ul>\n";
	    $lastcompany = $company;
	}
	print THISMACHINE "<li><a href=\"../" . 
            ($game->serfile ? 
	     $game->serfile . "#" : 
	     &main::strip_name($company) . ".html#") . 
	    &main::strip_name($game->gamename) . "\">" . 
	    $game->gamename . "</a>\n";
    }
    print THISMACHINE "</ul>\n</ul>\n\n";
    &print_subpage_footer(THISMACHINE);
    close(THISMACHINE);
}
print MACHINES "</ul>\n\n";
&print_subpage_footer(MACHINES);
close(MACHINES);

open(AUTHORS, ">$htmldir/authors.html") ||
    die("Couldn't open $htmldir/authors.html!");
print AUTHORS "<!-- This file generated by gamelist.pl -->\n";
#print AUTHORS "<insert file=defines>\n";
print AUTHORS "<html><head><title>Adventureland: Author Index</title></head>\n";
print_page_header(AUTHORS);
print AUTHORS "<h1><font color=\"#FF0000\">Author Index</font></h1>\n";
print AUTHORS "<ul>\n";
foreach $author (sort(keys(%authors))) {
    print AUTHORS "<li>$author\n<ul>";
    foreach $game (@{$authors{$author}}) {
	print AUTHORS "<li><a href=\"" . 
	    &strip_name($game->company) .
	    ".html\">" .
	    $game->company . 
	    "</a>: <a href=\"" .
	    &strip_name($game->company) .
	    ".html#" .
	    &strip_name($game->gamename) .
	    "\">" .
	    $game->gamename .
	    "</a>\n";
    }
    print AUTHORS "</ul>\n";
}
print AUTHORS "</ul>\n";
&print_page_footer(AUTHORS);
close(AUTHORS);

open(SYSTEMINDEX, ">$htmldir/systems/index.html") ||
    die("Couldn't open $htmldir/systems/index.html!");
print SYSTEMINDEX "<!-- This file generated by gamelist.pl -->\n";
#print SYSTEMINDEX "<insert file=../defines>\n";
print SYSTEMINDEX "<html><head><title>Adventureland: Game System Index</title>\n";
print_subpage_header(SYSTEMINDEX);
print SYSTEMINDEX "<h1><font color=\"#FF0000\">Game System Index</font></h1>\n";
print SYSTEMINDEX "<ul>\n";
foreach $platform (sort(keys(%platforms))) {
    if ($platforms{$platform}->filename ne "") {
	if ($system_open) {
	    print SYSTEM "</ul>\n";
	    &print_subpage_footer(SYSTEM);
	    close(SYSTEM);
	}
	$filename = "$htmldir/systems/" . 
	    &strip_name($platforms{$platform}->filename) .
	    ".html";
	open(SYSTEM, ">$filename") ||
	    die("Couldn't open $filename!");
	$system_open = 1;
	print SYSTEM "<!-- This file generated by gamelist.pl -->\n";
	#print SYSTEM "<insert file=../defines>\n";
	print SYSTEM "<html><head><title>Adventureland: " .
	    $platforms{$platform}->filename . 
	    "</title>\n";
        print_subpage_header(SYSTEM);
	print SYSTEM "<h1>" . $platforms{$platform}->filename . "</h1>\n";
	print SYSTEM "<ul>\n";
	print SYSTEMINDEX "<li><a href=\"" .
	    &strip_name($platforms{$platform}->filename) .
	    ".html\">" .
	    $platforms{$platform}->filename .
	    "</a>\n";
    }
    print SYSTEM "<li>$platform\n";
    print SYSTEM "<br>" . $platforms{$platform}->platdesc . "\n";
    foreach $emulator (keys(%emulators)) {
	if (grep(/$platform/, $emulators{$emulator}->plays)) {
	    print SYSTEM "$platform can be played with " .
		"<a href=\"../interpreters/" .
		&strip_name($emulator) .
		".html\">$emulator</a>.\n";
	}
    }
}
print SYSTEMINDEX "</ul>\n";
&print_subpage_footer(SYSTEMINDEX);
close(SYSTEMINDEX);
print SYSTEM "</ul>\n";
&print_subpage_footer(SYSTEM);
close(SYSTEM);

open(INTERPS, ">$htmldir/interpreters/index.html") ||
    die("Couldn't open $htmldir/interpreters/index.html!");
print INTERPS "<!-- This file generated by gamelist.pl -->\n";
#print INTERPS "<insert file=../defines>\n";
print INTERPS "<html><head><title>Adventureland: Interpreter Index</title>\n";
print_subpage_header(INTERPS);
print INTERPS "<h1><font color=\"#FF0000\">Interpreter Index</font></h1>\n";
print INTERPS "<ul>\n";
foreach $emulator (keys(%emulators)) {
    print INTERPS "<li><a href=\"" .
	&strip_name($emulator) .
	".html\">$emulator</a>\n";
    $filename = "$htmldir/interpreters/" . &strip_name($emulator) . ".html";
    open(EMULATOR, ">$filename") ||
	die("Couldn't open $filename!");
    print EMULATOR "<!-- This file generated by gamelist.pl -->\n";
#    print EMULATOR "<insert file=../defines>\n";
    print EMULATOR "<html><head><title>Adventureland: $emulator</title>\n";
print_subpage_header(EMULATOR);
    print EMULATOR "<h1>$emulator</h1>\n";
    print EMULATOR "<p>" . $emulators{$emulator}->emuldesc . "\n"
	if ($emulators{$emulator}->emuldesc);
    print EMULATOR "<p>$emulator can be downloaded from <a href=\"" .
	$emulators{$emulator}->emulhome . 
	"\">" .
	$emulators{$emulator}->emulhome . 
	"</a>\n"
	    if ($emulators{$emulator}->emulhome);
    if ($emulators{$emulator}->plays) {
	print EMULATOR "<p>$emulator plays games for these <a href=\"../systems/\">systems</a>:\n<ul>\n";
	foreach $plays ($emulators{$emulator}->plays) {
	    print EMULATOR "<li>$plays\n";
	}
	print EMULATOR "</ul>\n";
    }
    print EMULATOR "</ul>\n";
    &print_subpage_footer(EMULATOR);
    close(EMULATOR);
}
print INTERPS "</ul>\n";
&print_subpage_footer(INTERPS);
close(INTERPS);

